@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -24,6 +24,7 @@
 #if !defined(__SLEPCST_H)
 #define __SLEPCST_H
 #include <slepcsys.h>
+#include <slepcbv.h>
 #include <petscksp.h>
 
 PETSC_EXTERN PetscErrorCode STInitializePackage(void);
@@ -50,7 +51,6 @@ typedef const char* STType;
 #define STSHIFT     "shift"
 #define STSINVERT   "sinvert"
 #define STCAYLEY    "cayley"
-#define STFOLD      "fold"
 #define STPRECOND   "precond"
 
 /* Logging support */
@@ -63,6 +63,7 @@ PETSC_EXTERN PetscErrorCode STSetType(ST,STType);
 PETSC_EXTERN PetscErrorCode STGetType(ST,STType*);
 PETSC_EXTERN PetscErrorCode STSetOperators(ST,PetscInt,Mat*);
 PETSC_EXTERN PetscErrorCode STGetOperators(ST,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode STGetTOperators(ST,PetscInt,Mat*);
 PETSC_EXTERN PetscErrorCode STGetNumMatrices(ST,PetscInt*);
 PETSC_EXTERN PetscErrorCode STSetUp(ST);
 PETSC_EXTERN PetscErrorCode STSetFromOptions(ST);
@@ -71,11 +72,12 @@ PETSC_EXTERN PetscErrorCode STView(ST,PetscViewer);
 PETSC_EXTERN PetscErrorCode STApply(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STMatMult(ST,PetscInt,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STMatMultTranspose(ST,PetscInt,Vec,Vec);
-PETSC_EXTERN PetscErrorCode STMatSolve(ST,PetscInt,Vec,Vec);
-PETSC_EXTERN PetscErrorCode STMatSolveTranspose(ST,PetscInt,Vec,Vec);
+PETSC_EXTERN PetscErrorCode STMatSolve(ST,Vec,Vec);
+PETSC_EXTERN PetscErrorCode STMatSolveTranspose(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STGetBilinearForm(ST,Mat*);
 PETSC_EXTERN PetscErrorCode STApplyTranspose(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STComputeExplicitOperator(ST,Mat*);
+PETSC_EXTERN PetscErrorCode STMatSetUp(ST,PetscScalar,PetscScalar*);
 PETSC_EXTERN PetscErrorCode STPostSolve(ST);
 
 PETSC_EXTERN PetscErrorCode STSetKSP(ST,KSP);
@@ -83,8 +85,11 @@ PETSC_EXTERN PetscErrorCode STGetKSP(ST,KSP*);
 PETSC_EXTERN PetscErrorCode STSetShift(ST,PetscScalar);
 PETSC_EXTERN PetscErrorCode STGetShift(ST,PetscScalar*);
 PETSC_EXTERN PetscErrorCode STSetDefaultShift(ST,PetscScalar);
+PETSC_EXTERN PetscErrorCode STScaleShift(ST,PetscScalar);
 PETSC_EXTERN PetscErrorCode STSetBalanceMatrix(ST,Vec);
 PETSC_EXTERN PetscErrorCode STGetBalanceMatrix(ST,Vec*);
+PETSC_EXTERN PetscErrorCode STSetTransform(ST,PetscBool);
+PETSC_EXTERN PetscErrorCode STGetTransform(ST,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode STSetOptionsPrefix(ST,const char*);
 PETSC_EXTERN PetscErrorCode STAppendOptionsPrefix(ST,const char*);
@@ -92,10 +97,11 @@ PETSC_EXTERN PetscErrorCode STGetOptionsPrefix(ST,const char*[]);
 
 PETSC_EXTERN PetscErrorCode STBackTransform(ST,PetscInt,PetscScalar*,PetscScalar*);
 
-PETSC_EXTERN PetscErrorCode STCheckNullSpace(ST,PetscInt,const Vec[]);
+PETSC_EXTERN PetscErrorCode STCheckNullSpace(ST,BV);
 
-PETSC_EXTERN PetscErrorCode STGetOperationCounters(ST,PetscInt*,PetscInt*);
-PETSC_EXTERN PetscErrorCode STResetOperationCounters(ST);
+PETSC_EXTERN PetscErrorCode STMatCreateVecs(ST,Vec*,Vec*);
+PETSC_EXTERN PetscErrorCode STMatGetSize(ST,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode STMatGetLocalSize(ST,PetscInt*,PetscInt*);
 
 /*E
     STMatMode - Determines how to handle the coefficient matrix associated
@@ -114,8 +120,6 @@ PETSC_EXTERN PetscErrorCode STSetMatStructure(ST,MatStructure);
 PETSC_EXTERN PetscErrorCode STGetMatStructure(ST,MatStructure*);
 
 PETSC_EXTERN PetscFunctionList STList;
-PETSC_EXTERN PetscBool         STRegisterAllCalled;
-PETSC_EXTERN PetscErrorCode STRegisterAll(void);
 PETSC_EXTERN PetscErrorCode STRegister(const char[],PetscErrorCode(*)(ST));
 
 /* --------- options specific to particular spectral transformations-------- */
@@ -129,10 +133,10 @@ PETSC_EXTERN PetscErrorCode STShellSetBackTransform(ST st,PetscErrorCode (*backt
 PETSC_EXTERN PetscErrorCode STCayleyGetAntishift(ST,PetscScalar*);
 PETSC_EXTERN PetscErrorCode STCayleySetAntishift(ST,PetscScalar);
 
-PETSC_EXTERN PetscErrorCode STPrecondGetMatForPC(ST st,Mat *mat);
-PETSC_EXTERN PetscErrorCode STPrecondSetMatForPC(ST st,Mat mat);
-PETSC_EXTERN PetscErrorCode STPrecondGetKSPHasMat(ST st,PetscBool *setmat);
-PETSC_EXTERN PetscErrorCode STPrecondSetKSPHasMat(ST st,PetscBool setmat);
+PETSC_EXTERN PetscErrorCode STPrecondGetMatForPC(ST,Mat*);
+PETSC_EXTERN PetscErrorCode STPrecondSetMatForPC(ST,Mat);
+PETSC_EXTERN PetscErrorCode STPrecondGetKSPHasMat(ST,PetscBool*);
+PETSC_EXTERN PetscErrorCode STPrecondSetKSPHasMat(ST,PetscBool);
 
 #endif
 
