@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2013, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
 
@@ -19,18 +19,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include <slepc-private/slepcimpl.h>           /*I "slepcsys.h" I*/
-#include <slepc-private/epsimpl.h>
-#include <slepc-private/stimpl.h>
-#include <slepc-private/svdimpl.h>
-#include <slepc-private/qepimpl.h>
-#include <slepc-private/nepimpl.h>
-#include <slepc-private/mfnimpl.h>
-#include <slepc-private/ipimpl.h>
-#include <slepc-private/dsimpl.h>
-#include <slepc-private/fnimpl.h>
-#include <slepc-private/vecimplslepc.h>
-#include <stdlib.h>
+#include <slepc/private/slepcimpl.h>           /*I "slepcsys.h" I*/
+#include <slepc/private/vecimplslepc.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "SlepcGetVersion"
@@ -46,7 +36,7 @@
     Fortran Note:
     This routine is not supported in Fortran.
 
-    Level: developer
+    Level: intermediate
 @*/
 PetscErrorCode SlepcGetVersion(char version[],size_t len)
 {
@@ -68,7 +58,7 @@ PetscErrorCode SlepcGetVersion(char version[],size_t len)
 
    Collective on MPI_Comm
 */
-PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
+static PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 {
   PetscErrorCode ierr;
   char           version[256];
@@ -90,7 +80,7 @@ PetscErrorCode SlepcPrintVersion(MPI_Comm comm)
 
    Collective on MPI_Comm
 */
-PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
+static PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 {
   PetscErrorCode  ierr;
 
@@ -109,7 +99,25 @@ PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 PetscBool SlepcBeganPetsc = PETSC_FALSE;
 PetscBool SlepcInitializeCalled = PETSC_FALSE;
 
-#if defined(PETSC_USE_DYNAMIC_LIBRARIES)
+#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
+
+#undef __FUNCT__
+#define __FUNCT__ "SlepcLoadDynamicLibrary"
+static PetscErrorCode SlepcLoadDynamicLibrary(const char *name,PetscBool *found)
+{
+  char           libs[PETSC_MAX_PATH_LEN],dlib[PETSC_MAX_PATH_LEN];
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscStrcpy(libs,SLEPC_LIB_DIR);CHKERRQ(ierr);
+  ierr = PetscStrcat(libs,"/libslepc");CHKERRQ(ierr);
+  ierr = PetscStrcat(libs,name);CHKERRQ(ierr);
+  ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,found);CHKERRQ(ierr);
+  if (*found) {
+    ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,dlib);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__
 #define __FUNCT__ "SlepcInitialize_DynamicLibraries"
@@ -121,35 +129,71 @@ PetscErrorCode SlepcInitialize_DynamicLibraries(void)
 {
   PetscErrorCode ierr;
   PetscBool      found;
-  char           libs[PETSC_MAX_PATH_LEN],dlib[PETSC_MAX_PATH_LEN];
+  PetscBool      preload;
 
   PetscFunctionBegin;
-  ierr = PetscStrcpy(libs,SLEPC_LIB_DIR);CHKERRQ(ierr);
-  ierr = PetscStrcat(libs,"/libslepc");CHKERRQ(ierr);
-  ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,&found);CHKERRQ(ierr);
-  if (found) {
-    ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,libs);CHKERRQ(ierr);
-  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries");
+  preload = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(NULL,NULL,"-dynamic_library_preload",&preload,NULL);CHKERRQ(ierr);
+  if (preload) {
+#if defined(PETSC_USE_SINGLE_LIBRARY)
+    ierr = SlepcLoadDynamicLibrary("",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+#else
+    ierr = SlepcLoadDynamicLibrary("sys",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+    ierr = SlepcLoadDynamicLibrary("eps",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+    ierr = SlepcLoadDynamicLibrary("pep",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+    ierr = SlepcLoadDynamicLibrary("nep",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+    ierr = SlepcLoadDynamicLibrary("svd",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+    ierr = SlepcLoadDynamicLibrary("mfn",&found);CHKERRQ(ierr);
+    if (!found) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to locate SLEPc dynamic library\nYou cannot move the dynamic libraries!");
+#endif
+  }
+
+#if defined(PETSC_HAVE_THREADSAFETY)
+  ierr = STInitializePackage();CHKERRQ(ierr);
+  ierr = DSInitializePackage();CHKERRQ(ierr);
+  ierr = FNInitializePackage();CHKERRQ(ierr);
+  ierr = BVInitializePackage();CHKERRQ(ierr);
+  ierr = RGInitializePackage();CHKERRQ(ierr);
+  ierr = EPSInitializePackage();CHKERRQ(ierr);
+  ierr = SVDInitializePackage();CHKERRQ(ierr);
+  ierr = PEPInitializePackage();CHKERRQ(ierr);
+  ierr = NEPInitializePackage();CHKERRQ(ierr);
+  ierr = MFNInitializePackage();CHKERRQ(ierr);
+#endif
   PetscFunctionReturn(0);
 }
 #endif
 
 #undef __FUNCT__
-#define __FUNCT__ "SlepcInitialize_LogEvents"
-/*
-    SlepcInitialize_LogEvents - Initialize log events not pertaining to any object class.
-*/
-PetscErrorCode SlepcInitialize_LogEvents(void)
+#define __FUNCT__ "SlepcCitationsInitialize"
+PetscErrorCode SlepcCitationsInitialize()
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscLogEventRegister("UpdateVectors",0,&SLEPC_UpdateVectors);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMAXPBY",0,&SLEPC_VecMAXPBY);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DenseMatProd",EPS_CLASSID,&SLEPC_SlepcDenseMatProd);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DenseMatNorm",EPS_CLASSID,&SLEPC_SlepcDenseNorm);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("DenseCopy",EPS_CLASSID,&SLEPC_SlepcDenseCopy);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecsMult",EPS_CLASSID,&SLEPC_VecsMult);CHKERRQ(ierr);
+  ierr = PetscCitationsRegister("@Article{slepc-toms,\n"
+    "   author = \"Vicente Hernandez and Jose E. Roman and Vicente Vidal\",\n"
+    "   title = \"{SLEPc}: A Scalable and Flexible Toolkit for the Solution of Eigenvalue Problems\",\n"
+    "   journal = \"{ACM} Trans. Math. Software\",\n"
+    "   volume = \"31\",\n"
+    "   number = \"3\",\n"
+    "   pages = \"351--362\",\n"
+    "   year = \"2005,\"\n"
+    "   doi = \"http://dx.doi.org/10.1145/1089014.1089019\"\n"
+    "}\n",NULL);CHKERRQ(ierr);
+  ierr = PetscCitationsRegister("@TechReport{slepc-manual,\n"
+    "   author = \"J. E. Roman and C. Campos and E. Romero and A. Tomas\",\n"
+    "   title = \"{SLEPc} Users Manual\",\n"
+    "   number = \"DSIC-II/24/02 - Revision 3.7\",\n"
+    "   institution = \"D. Sistemes Inform\\`atics i Computaci\\'o, Universitat Polit\\`ecnica de Val\\`encia\",\n"
+    "   year = \"2016\"\n"
+    "}\n",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -190,10 +234,11 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const ch
     SlepcBeganPetsc = PETSC_TRUE;
   }
 
-#if defined(PETSC_USE_DYNAMIC_LIBRARIES)
+  ierr = SlepcCitationsInitialize();CHKERRQ(ierr);
+
+#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
   ierr = SlepcInitialize_DynamicLibraries();CHKERRQ(ierr);
 #endif
-  ierr = SlepcInitialize_LogEvents();CHKERRQ(ierr);
 
 #if defined(PETSC_HAVE_DRAND48)
   /* work-around for Cygwin drand48() initialization bug */
@@ -219,15 +264,15 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,const char file[],const ch
 @*/
 PetscErrorCode SlepcFinalize(void)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode ierr = 0;
 
   PetscFunctionBegin;
   ierr = PetscInfo(0,"SlepcFinalize() called\n");CHKERRQ(ierr);
   if (SlepcBeganPetsc) {
-    ierr = PetscFinalize();CHKERRQ(ierr);
+    ierr = PetscFinalize();
   }
   SlepcInitializeCalled = PETSC_FALSE;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(ierr);
 }
 
 #undef __FUNCT__
@@ -297,35 +342,3 @@ PetscErrorCode SlepcInitializeNoPointers(int argc,char **args,const char *filena
   PetscFunctionReturn(ierr);
 }
 
-#if defined(PETSC_USE_DYNAMIC_LIBRARIES)
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryRegister_slepc"
-/*
-  PetscDLLibraryRegister - This function is called when the dynamic library
-  it is in is opened.
-
-  This one registers all the EPS and ST methods in the libslepc.a
-  library.
-
-  Input Parameter:
-  path - library path
- */
-PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_slepc(char *path)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = EPSInitializePackage();CHKERRQ(ierr);
-  ierr = SVDInitializePackage();CHKERRQ(ierr);
-  ierr = QEPInitializePackage();CHKERRQ(ierr);
-  ierr = NEPInitializePackage();CHKERRQ(ierr);
-  ierr = MFNInitializePackage();CHKERRQ(ierr);
-  ierr = STInitializePackage();CHKERRQ(ierr);
-  ierr = IPInitializePackage();CHKERRQ(ierr);
-  ierr = DSInitializePackage();CHKERRQ(ierr);
-  ierr = FNInitializePackage();CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-#endif /* PETSC_USE_DYNAMIC_LIBRARIES */
-
